>From a445cf989fe4421a1ff27cc60abb261595ed0a49 Mon Sep 17 00:00:00 2001
From: Andres Freund <andres@anarazel.de>
Date: Wed, 10 Sep 2014 16:51:04 +0200
Subject: [PATCH] Add support for optional_argument to our own getopt_long()
 implementation.

07c8651dd91d5a currently causes compilation errors on mscv (and
probably some other) compilers because our getopt_long()
implementation doesn't have support for optional_argument.

Thus implement optional_argument in our fallback implemenation. It's
quite possibly also useful in other cases.

Arguably this needs a configure check for optional_argument, but it
has existed pretty much since getopt_long() was introduced and thus
doesn't seem worth the configure runtime.

Author: Michael Paquier and Andres Freund

Discussion: CAB7nPqS5VeedSCxrK=QouokbawgGKLpyc1Q++RRFCa_sjcSVrg@mail.gmail.com
---
 src/include/getopt_long.h |  1 +
 src/port/getopt_long.c    | 16 ++++++++++++----
 2 files changed, 13 insertions(+), 4 deletions(-)

diff --git a/src/include/getopt_long.h b/src/include/getopt_long.h
index e23c21f..71a884c 100644
--- a/src/include/getopt_long.h
+++ b/src/include/getopt_long.h
@@ -23,6 +23,7 @@ struct option
 
 #define no_argument 0
 #define required_argument 1
+#define optional_argument 2
 #endif
 
 #ifndef HAVE_GETOPT_LONG
diff --git a/src/port/getopt_long.c b/src/port/getopt_long.c
index b099091..aa5b731 100644
--- a/src/port/getopt_long.c
+++ b/src/port/getopt_long.c
@@ -100,11 +100,14 @@ getopt_long(int argc, char *const argv[],
 				if (strlen(longopts[i].name) == namelen
 					&& strncmp(place, longopts[i].name, namelen) == 0)
 				{
-					if (longopts[i].has_arg)
+					int			has_arg = longopts[i].has_arg;
+
+					if (has_arg != no_argument)
 					{
 						if (place[namelen] == '=')
 							optarg = place + namelen + 1;
-						else if (optind < argc - 1)
+						else if (optind < argc - 1 &&
+								 has_arg == required_argument)
 						{
 							optind++;
 							optarg = argv[optind];
@@ -113,13 +116,18 @@ getopt_long(int argc, char *const argv[],
 						{
 							if (optstring[0] == ':')
 								return BADARG;
-							if (opterr)
+
+							if (opterr && has_arg == required_argument)
 								fprintf(stderr,
 								   "%s: option requires an argument -- %s\n",
 										argv[0], place);
+
 							place = EMSG;
 							optind++;
-							return BADCH;
+
+							if (has_arg == required_argument)
+								return BADCH;
+							optarg = NULL;
 						}
 					}
 					else
-- 
2.0.0.rc2.4.g1dc51c6.dirty

